VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "CommonFileDialog"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

'-----------------------------------------------------------------
'  CommonDialogs for File Open/Save
'-----------------------------------------------------------------


Private OFN As OPENFILENAME

Private SelFileTitles As Collection
Private SelFileNames As Collection
Private SelFullName As String
Private SelFileTitle As String
Private SelPath As String
Private SelExtension As String


'-----------------------------------------------------------------
' Settings

Private Const PATH_SEP As String = "\"
Private Const PIPE As String = "|"

' This MAX_BUFFER value allows you to select approx.
' 500 files with an average length of 25 characters.
' Change this value as needed.
Private Const MAX_PATH As Long = 260 ' Max path length in windows
Private Const MAX_BUFFER As Long = 50 * MAX_PATH



'-----------------------------------------------------------------
' API Declarations

' OpenFileName constants
Private Enum CommonFileDialogFlags
  OFN_ALLOWMULTISELECT = &H200
  OFN_CREATEPROMPT = &H2000
  OFN_EXPLORER = &H80000
  OFN_EXTENSIONDIFFERENT = &H400
  OFN_FILEMUSTEXIST = &H1000
  OFN_HIDEREADONLY = &H4
  OFN_LONGNAMES = &H200000
  OFN_NOCHANGEDIR = &H8
  OFN_NODEREFERENCELINKS = &H100000
  OFN_OVERWRITEPROMPT = &H2
  OFN_PATHMUSTEXIST = &H800
  OFN_READONLY = &H1
End Enum


' API functions to use the Windows common dialog boxes.
Private Declare Function GetOpenFileName Lib "comdlg32" Alias "GetOpenFileNameA" (pOpenfilename As OPENFILENAME) As Long
Private Declare Function GetSaveFileName Lib "comdlg32" Alias "GetSaveFileNameA" (pOpenfilename As OPENFILENAME) As Long
Private Declare Function GetActiveWindow Lib "user32" () As Long
Private Declare Function CommDlgExtendedError Lib "comdlg32" () As Long
Private Declare Function IsWindow Lib "user32" (ByVal hWnd As Long) As Long

Private Type OPENFILENAME
  lStructSize       As Long
  hWndOwner         As Long
  hInstance         As Long
  lpstrFilter       As String
  lpstrCustomFilter As String
  nMaxCustFilter    As Long
  nFilterIndex      As Long
  lpstrFile         As String
  nMaxFile          As Long
  lpstrFileTitle    As String
  nMaxFileTitle     As Long
  lpstrInitialDir   As String
  lpstrTitle        As String
  Flags             As Long
  nFileOffset       As Integer
  nFileExtension    As Integer
  lpstrDefExt       As String
  lCustData         As Long
  lpfnHook          As Long
  lpTemplateName    As String  ' Can also be a Long.
End Type




'-----------------------------------------------------------------
' Initialize

Private Sub Class_Initialize()
  ' This routine runs when the object is created.
  OFN.Flags = OFN_EXPLORER Or OFN_LONGNAMES Or OFN_HIDEREADONLY Or OFN_FILEMUSTEXIST Or OFN_PATHMUSTEXIST
  OFN.lpstrFilter = "All Files (*.*)" & vbNullChar & "*.*" & vbNullChar & vbNullChar
End Sub


'-----------------------------------------------------------------
' Default Properties

Public Property Get AllowMultiSelect() As Boolean
  AllowMultiSelect = Flag(OFN_ALLOWMULTISELECT)
End Property

Property Let AllowMultiSelect(NewValue As Boolean)
  Flag(OFN_ALLOWMULTISELECT) = NewValue
End Property

Public Property Get DialogTitle() As String
  DialogTitle = OFN.lpstrTitle
End Property

Public Property Let DialogTitle(NewValue As String)
  OFN.lpstrTitle = NewValue
End Property


'-----------------------------------------------------------------
' Special: Filter

Public Property Let Filter(Filter As Variant)
  If IsArray(Filter) Then
    OFN.lpstrFilter = Join(Filter, vbNullChar) & vbNullChar
  Else
    OFN.lpstrFilter = Replace(Filter, PIPE, vbNullChar) & vbNullChar
  End If
End Property

Public Property Get Filter() As Variant
  ' Returned value is the selected item
  With OFN
    If .nFilterIndex Then
      Dim Temp() As String
      Temp = Split(.lpstrFilter, vbNullChar)
      Filter = Temp(.nFilterIndex * 2 - 2) & PIPE & Temp(.nFilterIndex * 2 - 1)
    End If
  End With
End Property

Public Property Let FilterIndex(NewValue As Long)
  OFN.nFilterIndex = NewValue
End Property

Public Property Get FilterIndex() As Long
  FilterIndex = OFN.nFilterIndex
End Property

'-----------------------------------------------------------------
' File Paths settings

Public Property Get UseDefaultDir() As Boolean
  UseDefaultDir = Flag(OFN_NOCHANGEDIR)
End Property

Public Property Let UseDefaultDir(NewValue As Boolean)
  Flag(OFN_NOCHANGEDIR) = NewValue
End Property

Public Property Get FileMustExist() As Boolean
  FileMustExist = Flag(OFN_FILEMUSTEXIST)
End Property

Public Property Let FileMustExist(NewValue As Boolean)
  Flag(OFN_FILEMUSTEXIST) = NewValue
End Property

Public Property Get PathMustExist() As Boolean
  PathMustExist = Flag(OFN_PATHMUSTEXIST)
End Property

Public Property Let PathMustExist(NewValue As Boolean)
  Flag(OFN_PATHMUSTEXIST) = NewValue
End Property

Public Property Get PromptCreate() As Boolean
  PromptCreate = Flag(OFN_CREATEPROMPT)
End Property

Public Property Let PromptCreate(NewValue As Boolean)
  Flag(OFN_CREATEPROMPT) = NewValue
End Property


'-----------------------------------------------------------------
' Read only checkbox

Public Property Get ReadOnlyCheckBox() As Boolean
  ReadOnlyCheckBox = Not Flag(OFN_HIDEREADONLY)
End Property

Public Property Let ReadOnlyCheckBox(NewValue As Boolean)
  Flag(OFN_HIDEREADONLY) = Not NewValue
End Property

Public Property Get ReadOnlyChecked() As Boolean
  ReadOnlyChecked = Flag(OFN_READONLY)
End Property

Public Property Let ReadOnlyChecked(NewValue As Boolean)
  Flag(OFN_READONLY) = NewValue
End Property

'-----------------------------------------------------------------
' File names

Public Property Get FileName() As String
  FileName = SelFullName
End Property

Public Property Get FileNames() As Collection
  Set FileNames = SelFileNames
End Property

Public Property Get FileTitle() As String
  FileTitle = SelFileTitle
End Property

Public Property Get FileTitles() As Collection
  Set FileTitles = SelFileTitles
End Property

Public Property Get Extension() As String
  Extension = SelExtension
End Property


'-----------------------------------------------------------------
' Default Filename Settings

Property Get DefaultDirectory() As String
  DefaultDirectory = OFN.lpstrInitialDir
End Property

Property Let DefaultDirectory(NewValue As String)
  OFN.lpstrInitialDir = NewValue
End Property

Public Property Get DefaultFileName() As String
  DefaultFileName = OFN.lpstrFile
End Property

Public Property Let DefaultFileName(NewValue As String)
  If Len(NewValue) <= MAX_PATH Then
    OFN.lpstrFile = NewValue
  Else
    Err.Raise 52 'Bad file name or number
  End If
End Property

Public Property Get DefaultExtension() As String
  DefaultExtension = OFN.lpstrDefExt
End Property

Public Property Let DefaultExtension(NewValue As String)
  OFN.lpstrDefExt = LCase(Left(Replace(NewValue, ".", vbNullString), 3))
End Property


'-----------------------------------------------------------------
' Owner

Property Get hWndParent() As Long
  hWndParent = OFN.hWndOwner
End Property

Property Let hWndParent(hWnd As Long)
  If IsWindow(hWnd) = 0 And hWnd <> 0 Then Error 380
  OFN.hWndOwner = hWnd
End Property


'-----------------------------------------------------------------
' Show the dialog

Function ShowOpen() As Boolean
  ShowOpen = Show(True)
End Function

Function ShowSave() As Boolean
  ' Set or clear appropriate flags for Save As dialog.
  ShowSave = Show(False)
End Function

Private Function Show(ShowOpen As Boolean)
  Dim OldhWnd As Long
  Dim OldFlags As CommonFileDialogFlags
  Dim OldFile As String
  
  With OFN
    .lStructSize = Len(OFN)
    
    OldhWnd = .hWndOwner
    OldFlags = .Flags
    OldFile = .lpstrFile
    
    If Not ShowOpen Then
      Flag(OFN_ALLOWMULTISELECT) = False
      Flag(OFN_PATHMUSTEXIST) = True
      Flag(OFN_OVERWRITEPROMPT) = True
    End If
    
    If .hWndOwner = 0 Or IsWindow(.hWndOwner) = 0 Then
      .hWndOwner = GetActiveWindow() ' Cool, do somethings for us
    End If
    
    If Flag(OFN_READONLY) Then Flag(OFN_HIDEREADONLY) = False ' Should be displayed then aswell
    
    ' Create large buffer if multiple file selection is allowed.
    If Flag(OFN_ALLOWMULTISELECT) Then
      .nMaxFile = MAX_BUFFER + 1
    Else
      .nMaxFile = MAX_PATH + 1
    End If
    
    .nMaxFileTitle = MAX_PATH + 1
    
    ' Initialize the buffers.
    .lpstrFile = .lpstrFile & String(.nMaxFile - 1 - Len(.lpstrFile), 0)
    .lpstrFileTitle = String(.nMaxFileTitle - 1, 0)
    
    ' Display the appropriate dialog.
    If ShowOpen Then
      Show = GetOpenFileName(OFN)
    Else
      Show = GetSaveFileName(OFN)
    End If
    
    If Show Then
      ' Remove trailing null characters.
      Dim EndPos As Long
      Dim EndTitlePos As Long
      
      EndPos = InStr(.lpstrFile & vbNullChar, (vbNullChar & vbNullChar))
      EndTitlePos = InStr(.lpstrFileTitle, vbNullChar)
      
      If EndPos Then
        SelFullName = Left(.lpstrFile, EndPos - 1)               ' Get the file name including the path name.
        SelFileTitle = Left(.lpstrFileTitle, EndTitlePos - 1)    ' Get the file name without the path name.
        SelPath = Left(SelFullName, .nFileOffset - 1)            ' Get the path name.
        If .nFileExtension Then
          SelExtension = Mid(SelFullName, .nFileExtension + 1)   ' Get the extension.
        End If
        
        Set SelFileTitles = New Collection
        Set SelFileNames = New Collection
        
        ' If sFileTitle is a string, we have a single selection.
        If Len(SelFileTitle) Then
          ' Add to the collections.
          SelFileTitles.Add Mid(SelFullName, .nFileOffset + 1)
          SelFileNames.Add SelFullName
        Else
          ' Tear multiple selection apart.
          Dim Files() As String
          Dim I As Long
          
          Files = Split(SelFullName, vbNullChar)
          
          If UBound(Files) > LBound(Files) Then
            ' We have more than one array element!
            
            ' Remove backslash if sPath is the root folder.
            If Len(SelPath) = 3 Then SelPath = Left(SelPath, 2)
            
            ' Note that the first element is the path name, thus omitted
            For I = 1 To UBound(Files)
              SelFileTitles.Add Files(I)
              ' If the string already contains a backslash,
              ' the user must have selected a shortcut file,
              ' so we don't add the path.
              If InStr(Files(I), PATH_SEP) Then
                SelFileNames.Add Files(I)
              Else
                SelFileNames.Add SelPath & PATH_SEP & Files(I)
              End If
            Next
            
            ' Clear this variable.
            SelFullName = vbNullString
          End If ' Ubound
        End If ' Multiple Files
        
        ' Always path separator at the end
        If Right(SelPath, Len(PATH_SEP)) <> PATH_SEP Then SelPath = SelPath & PATH_SEP
      End If ' EndPos
    Else
      Dim ErrorCode As Long
      ErrorCode = CommDlgExtendedError()
      If ErrorCode Then Err.Raise ErrorCode, "CommonFileDialog", "Common file dialog failed to open"
    End If ' Show
    
    .hWndOwner = OldhWnd
    .Flags = OldFlags
    .lpstrFile = OldFile
  End With
End Function


'-----------------------------------------------------------------
' Utilities

Private Property Get Flag(FlagConstants As CommonFileDialogFlags) As Boolean
  Flag = (OFN.Flags And FlagConstants) = FlagConstants
End Property

Private Property Let Flag(FlagConstants As CommonFileDialogFlags, NewValue As Boolean)
  With OFN
    If NewValue = True Then
      .Flags = .Flags Or FlagConstants
    Else
      .Flags = .Flags And Not FlagConstants
    End If
  End With
End Property

