Attribute VB_Name = "GameHelpers"
Option Explicit


'----------------------------------------------------------------
' Restore the structures are they were

Sub RestoreVariables()
  ' Just clears the status
  ' This should do anything with the loaded level itself.
  Game.PlayerStyle = STANDLEFT
  Game.StepsDone = 0
  Game.PrevStep = 0
  Game.Complete = IsComplete()
  Game.UndoAvailable = False
End Sub

'----------------------------------------------------------------
' Clear the level, or initialize for editing modes

Sub InitDefaultEditLevel()
  Call ClearLevel(True)
  Game.FileName = ""
End Sub

Sub ClearLevel(ShowEditSample As Boolean)
  Dim X As Integer
  Dim Y As Integer
  
  Call RestoreVariables
  Game.PlayedLevel.PLAYER.X = 1
  Game.PlayedLevel.PLAYER.Y = 1
  
  If Not ShowEditSample Then
    ' Cleat the matrix by redimensioning it
    ReDim Game.PlayedLevel.Fields(0 To 8, 0 To 7) As Byte
    Game.PlayedLevel.Width = UBound(Game.PlayedLevel.Fields, 1) + 1
    Game.PlayedLevel.Height = UBound(Game.PlayedLevel.Fields, 2) + 1
    
    Game.PlayedLevel.Name = ""
    Game.PlayedLevel.OtherInfo = ""
  Else
    ' Make an same level (wall border around it)
    Game.PlayedLevel.Name = "Untitled"
    For X = 0 To Game.PlayedLevel.Width - 1
      Game.PlayedLevel.Fields(X, 0) = WALL
      Game.PlayedLevel.Fields(X, Game.PlayedLevel.Height - 1) = WALL
    Next
    For Y = 1 To Game.PlayedLevel.Height - 2
      Game.PlayedLevel.Fields(0, Y) = WALL
      Game.PlayedLevel.Fields(Game.PlayedLevel.Width - 1, Y) = WALL
      For X = 1 To Game.PlayedLevel.Width - 2
        Game.PlayedLevel.Fields(X, Y) = SPACE
      Next
    Next
  End If
  
  ' Load it
  Call LoadEditedLevel
End Sub


'----------------------------------------------------------------
' Is Level Complete?

Public Function IsComplete() As Boolean
  Dim X As Integer
  Dim Y As Integer
  With Game.PlayedLevel
    For X = 0 To .Width - 1
      For Y = 0 To .Height - 1
        If .Fields(X, Y) = TARGET Or .Fields(X, Y) = BOX Then
          IsComplete = False
          Exit Function
        End If
      Next
    Next
  End With
  
  IsComplete = True
End Function


'----------------------------------------------------------------
' Level Matrix Copiers

Sub RestoreLevel()
  ' Original Matrix is stored in the OriginalLevel variable
  LSet Game.PlayedLevel = Game.OriginalLevel
End Sub

Sub LoadEditedLevel()
  ' The modified matrix is stored in the Original variable
  LSet Game.OriginalLevel = Game.PlayedLevel
End Sub



'----------------------------------------------------------------
' Save to the undo buffer

Public Sub SaveUndo(CurrentPos As LOCATION, PosTo As LOCATION, PosAfterTo As LOCATION)
  With Game.Undo(0)
    .PlayerStyle = Game.PlayerStyle
    .PlayerPos = CurrentPos
    .NextPos = PosTo
    .NextValue = Game.PlayedLevel.Fields(PosTo.X, PosTo.Y)
    .NextAfterPos = PosAfterTo
    .NextAfterValue = Game.PlayedLevel.Fields(PosAfterTo.X, PosAfterTo.Y)
  End With
  Game.UndoAvailable = True
End Sub

