Attribute VB_Name = "LevelSaver"
Option Explicit

'----------------------------------------------------------------
' Save the level

Public Function SaveLevel(Optional DialogType As Integer) As Boolean
  Dim FileData As LEVELFILE
  Dim Handle As Integer
  Dim NewFields() As Byte
  Dim X As Integer
  Dim Y As Integer
  Dim NewWidth As Integer
  Dim NewHeight As Integer
  Dim LineEmpty As Boolean
  
  ' Test the player position
  If Game.PlayedLevel.Fields(Game.PlayedLevel.Player.X, Game.PlayedLevel.Player.Y) <> SPACE Then
    MsgBox "The start position of the player should be set in a empty location!", vbExclamation
    Exit Function
  End If
  
  ' Test the level name
  If Game.PlayedLevel.Name = "" Then
    MsgBox "You need to enter a name for the level!", vbInformation
    Exit Function
  End If
  
  ' Show a dialog for empty filename, or for SaveAs
  If Game.FileName = "" Or DialogType = SAVE_AS Or Not FileExist(Game.FileName) Then
    Dim Dialog As New CommonFileDialog
    With Dialog
      .DialogTitle = "Save Level"
      If DialogType = SAVE_AS Then
        .DialogTitle = "Save Level As"
      Else
        .DialogTitle = "Save Level"
      End If
      .DefaultFileName = FormatDialogFile(Game.PlayedLevel.Name)
      .DefaultDirectory = DATAPATH & "Levels"
      .DefaultExtension = "car"
      .Filter = "Carrier Levels (*.car)|*.car"
      If Not ActiveForm Is Nothing Then .hWndParent = ActiveForm.hWnd
      If Not .ShowSave() Then Exit Function
      Game.FileName = .FileName
    End With
  End If
  
  ' Make the header structure
  FileData.ID = PROGRAM_ID
  FileData.VersionMajor = App.Major
  FileData.VersionMinor = App.Minor
  FileData.VersionRevision = App.Revision
  LSet FileData.Contents = Game.PlayedLevel
  FileData.Contents.Width = UBound(FileData.Contents.Fields, 1) + 1
  FileData.Contents.Height = UBound(FileData.Contents.Fields, 2) + 1
  
  ' Final test for bounds (bug prevention)
  If LBound(FileData.Contents.Fields, 1) <> 0 Or LBound(FileData.Contents.Fields, 2) <> 0 Then
    MsgBox "Internal Error due LBounds. Failed to Save!", vbCritical
    Exit Function
  End If
  
  ' Try clearing some edge of the level...
  NewWidth = FileData.Contents.Width
  NewHeight = FileData.Contents.Height
  LineEmpty = True
  For X = FileData.Contents.Width - 1 To 0 Step -1
    For Y = 0 To FileData.Contents.Height - 1
      If FileData.Contents.Fields(X, Y) <> SPACE Then
        LineEmpty = False
        Exit For
      End If
    Next
    If LineEmpty Then
      NewWidth = NewWidth - 1
    Else
      Exit For
    End If
  Next
  LineEmpty = True
  For Y = FileData.Contents.Height - 1 To 0 Step -1
    For X = 0 To NewWidth - 1
      If FileData.Contents.Fields(X, Y) <> SPACE Then
        LineEmpty = False
        Exit For
      End If
    Next
    If LineEmpty Then
      NewHeight = NewHeight - 1
    Else
      Exit For
    End If
  Next
  
  ' Whoa! The level can be resized.
  If NewWidth <> FileData.Contents.Width Or NewHeight <> FileData.Contents.Height Then
    ReDim NewFields(0 To NewWidth - 1, 0 To NewHeight - 1) As Byte
    For X = 0 To NewWidth - 1
      For Y = 0 To NewHeight - 1
        NewFields(X, Y) = FileData.Contents.Fields(X, Y)
      Next
    Next
    FileData.Contents.Fields = NewFields
    FileData.Contents.Width = NewWidth
    FileData.Contents.Height = NewHeight
  End If
  
  ' Dump the structure into the file
  Handle = FreeFile()
  Open Game.FileName For Binary Access Write Lock Read Write As #Handle
    Put #Handle, , FileData
  Close #Handle
  
  ' Done.
  SaveLevel = True
End Function

